package cz.fi.muni.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectWriter;
import com.fasterxml.jackson.databind.SerializationFeature;
import cz.fi.muni.production.model.NewBarrel;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;



import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(controllers = BarrelControllerTest.class)

public class BarrelControllerTest {
    @Autowired
    private MockMvc mockMvc;
    private final NewBarrel newBarrel = new NewBarrel();


    @Test
    void addBarrel() throws Exception {
        // if we will add a proper constructor this bit can be removed

        newBarrel.setAcidity(1);
        newBarrel.setAlcoholLevel(1.0);
        newBarrel.setSugarLevel(1.0);
        newBarrel.setVolume(1);

        mockMvc.perform(post("/barrel")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .content(objectToJson(newBarrel))
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isCreated());
    }

    @Test
    void getAllBarrels() throws Exception {
        mockMvc.perform(get("/barrels")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());
    }

    @Test
    void getBarrel() throws Exception{
        mockMvc.perform(get("/barrel/0")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());
    }

    @Test
    void editBarrel() throws Exception{
        mockMvc.perform(patch("/barrel/0")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .content(objectToJson(newBarrel))
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());
    }

    @Test
    void deleteBarrel() throws Exception {
        mockMvc.perform(delete("/barrel/0")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());

    }

    private static String objectToJson(Object object) throws JsonProcessingException {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(SerializationFeature.WRAP_ROOT_VALUE, false);
        mapper.configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false);
        ObjectWriter ow = mapper.writer().withDefaultPrettyPrinter();
        return ow.writeValueAsString(object);
    }
}
