package cz.fi.muni.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectWriter;
import com.fasterxml.jackson.databind.SerializationFeature;
import cz.fi.muni.production.model.NewIngredient;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(controllers = IngredientControllerTest.class)
public class IngredientControllerTest {
    @Autowired
    private MockMvc mockMvc;
    private final NewIngredient newIngredient = new NewIngredient();
    @Test
    void addIngredient() throws Exception {
        // again proper constructor would be better
        newIngredient.setName("test");
        newIngredient.setAmount(0.0);
        newIngredient.setPrice(0.0);

        mockMvc.perform(post("/ingredient")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .content(objectToJson(newIngredient))
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isCreated());
    }

    @Test
    void getAllIngredients() throws Exception {
        mockMvc.perform(get("/ingredients")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());
    }

    @Test
    void getIngredient() throws Exception {
        mockMvc.perform(get("/ingredient/0")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());
    }

    @Test
    void editIngredient() throws Exception {
        mockMvc.perform(patch("/ingredient/0")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .content(objectToJson(newIngredient))
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());
    }

    @Test
    void deleteIngredient() throws Exception {
        mockMvc.perform(delete("/ingredient/0")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());
    }

    private static String objectToJson(Object object) throws JsonProcessingException {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(SerializationFeature.WRAP_ROOT_VALUE, false);
        mapper.configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false);
        ObjectWriter ow = mapper.writer().withDefaultPrettyPrinter();
        return ow.writeValueAsString(object);
    }
}
