package cz.fi.muni.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectWriter;
import com.fasterxml.jackson.databind.SerializationFeature;
import cz.fi.muni.production.model.NewRecipe;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
@WebMvcTest(controllers = RecipeControllerTest.class)

class RecipeControllerTest {
    @Autowired
    private MockMvc mockMvc;
    private final NewRecipe newRecipe = new NewRecipe();

    @Test
    void addRecipe() throws Exception {
        // if we will add a proper constructor this bit can be removed
        newRecipe.setId(0);
        newRecipe.setTechnicalProcess("test");
        newRecipe.setGrapeWineRatio(1.0);
        newRecipe.setAgingTime(42);

        mockMvc.perform(post("/recipe")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .content(objectToJson(newRecipe))
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isCreated());
    }

    @Test
    void getAllRecipes() throws Exception {
        mockMvc.perform(get("/recipes")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());
    }

    @Test
    void getRecipe() throws Exception{
        mockMvc.perform(get("/recipe/0")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());
    }

    @Test
    void editRecipe() throws Exception{
        mockMvc.perform(patch("/recipe/0")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .content(objectToJson(newRecipe))
                        .accept(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());
    }

    private static String objectToJson(Object object) throws JsonProcessingException {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(SerializationFeature.WRAP_ROOT_VALUE, false);
        mapper.configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false);
        ObjectWriter ow = mapper.writer().withDefaultPrettyPrinter();
        return ow.writeValueAsString(object);
    }
}